@@ -5,7 +5,7 @@
  * pgpool: a language independent connection pool server for PostgreSQL
  * written by Tatsuo Ishii
  *
- * Copyright (c) 2003-2012	PgPool Global Development Group
+ * Copyright (c) 2003-2014	PgPool Global Development Group
  *
  * Permission to use, copy, modify, and distribute this software and
  * its documentation for any purpose and without fee is hereby
@@ -85,6 +85,9 @@ volatile sig_atomic_t exit_request = 0;
 static int idle;		/* non 0 means this child is in idle state */
 static int accepted = 0;
 
+static int child_inet_fd = 0;
+static int child_unix_fd = 0;
+
 extern int myargc;
 extern char **myargv;
 
@@ -131,6 +134,11 @@ void do_child(int unix_fd, int inet_fd)
 		pool_set_nonblock(inet_fd);
 	}
 #endif
+	if (inet_fd)
+	{
+		child_inet_fd = inet_fd;
+	}
+	child_unix_fd = unix_fd;
 
 	/* Initialize my backend status */
 	pool_initialize_private_backend_status();
@@ -376,12 +384,15 @@ void do_child(int unix_fd, int inet_fd)
 			{
 				/* client exits */
 				case POOL_END:
+				case POOL_END_WITH_FRONTEND_ERROR:
 					/*
 					 * do not cache connection if:
+					 * frontend abnormally exits or
 					 * pool_config->connection_cahe == 0 or
 					 * database name is template0, template1, postgres or regression
 					 */
-					if (pool_config->connection_cache == 0 ||
+					if (status == POOL_END_WITH_FRONTEND_ERROR ||
+						pool_config->connection_cache == 0 ||
 						!strcmp(sp->database, "template0") ||
 						!strcmp(sp->database, "template1") ||
 						!strcmp(sp->database, "postgres") ||
@@ -1279,6 +1290,14 @@ static RETSIGTYPE die(int sig)
 	switch (sig)
 	{
 		case SIGTERM:	/* smart shutdown */
+			/* Refuse further requests by closing listen socket */
+			if (child_inet_fd)
+			{
+				pool_log("die: close listen socket");
+				close(child_inet_fd);
+			}
+			close(child_unix_fd);
+
 			if (idle == 0)
 			{
 				pool_debug("child receives smart shutdown request but it's not in idle state");
@@ -2043,8 +2062,9 @@ static void init_system_db_connection(void)
 {	
 	if (pool_config->parallel_mode || pool_config->enable_query_cache)
 	{
-		system_db_connect();
-		if (PQstatus(system_db_info->pgconn) != CONNECTION_OK)
+		int nRet;
+		nRet = system_db_connect();
+		if (nRet || PQstatus(system_db_info->pgconn) != CONNECTION_OK)
 		{
 			pool_error("Could not make persistent libpq system DB connection");
 		}
