@@ -9,7 +9,7 @@
  * pgpool: a language independent connection pool server for PostgreSQL
  * written by Tatsuo Ishii
  *
- * Portions Copyright (c) 2003-2012	PgPool Global Development Group
+ * Portions Copyright (c) 2003-2014	PgPool Global Development Group
  * Portions Copyright (c) 1996-2005, PostgreSQL Global Development Group
  * Portions Copyright (c) 1994, Regents of the University of California
  *
@@ -47,9 +47,10 @@
 #include <stdlib.h>
 #include <string.h>
 #include <sys/un.h>
+
 #include "pool.h"
 #include "pool_ip.h"
-
+#include "pool_config.h"
 
 static int rangeSockAddrAF_INET(const struct sockaddr_in * addr,
 					 const struct sockaddr_in * netaddr,
@@ -463,6 +464,8 @@ SockAddr_cidr_mask(struct sockaddr_storage * mask, char *numbits, int family)
 				struct sockaddr_in6 mask6;
 				int			i;
 
+				memset(&mask6, 0, sizeof(mask6));
+
 				if (bits < 0 || bits > 128)
 					return -1;
 				for (i = 0; i < 16; i++)
