@@ -41,6 +41,7 @@ void pool_init_session_context(POOL_CONNECTION *frontend, POOL_CONNECTION_POOL *
 {
 	session_context = &session_context_d;
 	int node_id;
+	int i;
 
 	/* Get Process context */
 	session_context->process_context = pool_get_process_context();
@@ -76,10 +77,15 @@ void pool_init_session_context(POOL_CONNECTION *frontend, POOL_CONNECTION_POOL *
 		node_id = STREAM? PRIMARY_NODE_ID: MASTER_NODE_ID;
 	}
 
-	session_context->load_balance_node_id = 
-		process_info->connection_info->load_balancing_node = node_id;
+	session_context->load_balance_node_id = node_id;
 
-	pool_debug("selected load balancing node: %d", backend->info->load_balancing_node);
+	for (i=0;i<NUM_BACKENDS;i++)
+	{
+		pool_coninfo(session_context->process_context->proc_id,
+					 pool_pool_index(), i)->load_balancing_node = node_id;
+	}
+
+	pool_debug("selected load balancing node: %d", node_id);
 
 	/* Unset query is in progress */
 	pool_unset_query_in_progress();
